;		Accelerometer Gyro Leveler
;		PIC 16F62x series Micro
;		9k6 serial to SSC2
;		ADXL202 Accelerometer
;		EEPROM Reference Storage
;		11/22/2002		Tested Functional
;------------------------------------------------------------
        LIST    P=16F627
	ERRORLEVEL	-302
        include <P16F627.INC>
        __CONFIG        h'3d21'
;-------------------------------------------------------------
;               Port Set-up
;           0   1   2   3   4   5   6   7       bit
; port  A   In  In  Out Out In  X   X   X
;       B   Out Out Out Out Out Out Out Out
;------------------------------------------------------------
	ORG	0			; start code
	goto	main
;------------------------------------------------------------
        cblock  0x20
                t1_x                    ; 20
                t1_y                    ; 21
                x_cal                   ; 23
                y_cal                   ; 24
                count_1                 ; 28
                delay_count             ; 2a
                servo_1                 ; 2b
                servo_2                 ; 2c
		ee_adr
		ee_data
        endc
;------------------------------------------------------------
#define X_AXIS          PORTA,0
#define Y_AXIS          PORTA,1
#define	TOGGLE		PORTA,4
#define	LED		PORTB,7
;------------------------------------------------------------
main
        clrf    PORTA
        clrf    PORTB
        movlw   0x07
        movwf   CMCON                   ;compare module off
        bsf     STATUS,RP0              ;switch to bank 1
        movlw   b'00010011'             ;port A in/outputs
        movwf   TRISA
        movlw   b'00000110'             ;port B in/outputs
        movwf   TRISB
        movlw   b'10000100'             ;TMR0/32
        movwf   OPTION_REG
	movlw	d'80'
	movwf	PR2			;TMR2 setup
        movlw   b'00100100'
        movwf   TXSTA                   ;tx enabled, BRGH hi
        movlw   d'25'
        movwf   SPBRG                   ;9k6 serial
        bcf     STATUS,RP0              ;back to page 0
;
        clrf    PIR1                    ;reset perif int
        movlw   b'10000000'             ;serial enabled
        movwf   RCSTA                   ;
        movlw   b'00000000'             ;
	movwf	INTCON
	movlw	b'01111111'
	movwf	T2CON			;pres = 16, posts = 16
	bcf	LED			;LED off
;
;------------------------------------------------------------
	call	delay_256		;wait 256 mS
        call    read_x_axis
	call	read_y_axis
;
	btfss	TOGGLE
	goto	read_back
	movlw	d'0'
	movwf	ee_adr	
	movfw	t1_x
	movwf	ee_data
	call	ee_write
;
	movlw	d'1'
	movwf	ee_adr
	movfw	t1_y
	movwf	ee_data
	call	ee_write
;
read_back
	movlw	d'0'
	movwf	ee_adr
	call	ee_read
	movfw	ee_data
	movwf	x_cal
;
	movlw	d'1'
	movwf	ee_adr
	call	ee_read
	movfw	ee_data
	movwf	y_cal
;
	movlw	d'127'
	movwf	servo_1
	movwf	servo_2			;center servos
;
;------------------------------------------------------------
;		Timeout in 20.48 mS
check_accel
	btfss	PIR1,TMR2IF
	goto	$-1			;wait for servo timeout
	bcf	PIR1,TMR2IF
;	
;		Send serial position data for servo 1
servo_one
	movlw	0x00
	movwf	TXREG			;channel 0
	nop
        btfss   PIR1,TXIF
        goto    $-1
	movfw	servo_1	
	movwf	TXREG			;position data
	nop
        btfss   PIR1,TXIF
        goto    $-1
	movlw	0xff
	movwf	TXREG			;
	nop
        btfss   PIR1,TXIF
        goto    $-1	
;
servo_two
	movlw	0x01
	movwf	TXREG			;channel 1
	nop
        btfss   PIR1,TXIF
        goto    $-1
	movfw	servo_2
	movwf	TXREG			;position data
	nop
        btfss   PIR1,TXIF
        goto    $-1
	movlw	0xff
	movwf	TXREG			;
	nop
        btfss   PIR1,TXIF
        goto    $-1	
;
	bsf	LED			;LED on
        call    read_x_axis
	call	read_y_axis
        call    calc_accel
;
	bcf	LED			;LED off
        goto    check_accel
;------------------------------------------------------------
;               Subroutine area
;------------------------------------------------------------
;               Reads T1 X/Y, T2
read_x_axis
        btfsc   X_AXIS
        goto    $-1
        btfss   X_AXIS
        goto    $-1
        clrf    TMR0			;start timing
        btfsc   X_AXIS
        goto    $-1
        movfw   TMR0			;done timing X axis
        movwf   t1_x
	return
;
read_y_axis
        btfsc   Y_AXIS
        goto    $-1
        btfss   Y_AXIS
        goto    $-1
        clrf    TMR0			;start timing
        btfsc   Y_AXIS
        goto    $-1
        movfw   TMR0			;done timing Y axis
        movwf   t1_y
	return
;------------------------------------------------------------
calc_accel
	movfw	t1_x
	subwf	x_cal,W
	bz	check_y
	bnc	x_over
x_under
	incfsz	servo_1,W
	incf	servo_1,F
	goto	check_y	
x_over
	decfsz	servo_1,W
	decf	servo_1,F
	goto	check_y
check_y
	movfw	t1_y
	subwf	y_cal,W
	bz	were_done
	bnc	y_over
y_under
	incfsz	servo_2,W
	incf	servo_2,F
	goto	were_done	
y_over
	decfsz	servo_2,W
	decf	servo_2,F
were_done
	return
;------------------------------------------------------------
ee_write
	movlw	EEADR
	movwf	FSR
	movfw	ee_adr
	movwf	INDF
	movlw	EEDATA
	movwf	FSR
	movfw	ee_data
	movwf	INDF
;
	bsf	STATUS,RP0
	bsf	EECON1,WREN
	movlw	0x55
	movwf	EECON2
	movlw	0xaa
	movwf	EECON2
	bsf	EECON1,WR
	btfsc	EECON1,WR
	goto	$-1
	bcf	EECON1,WREN
	bcf	STATUS,RP0
	return
ee_read
	movlw	EEADR
	movwf	FSR
	movfw	ee_adr
	movwf	INDF
	bsf	STATUS,RP0
	bsf	EECON1,RD
	bcf	STATUS,RP0
	movlw	EEDATA
	movwf	FSR
	movfw	INDF
	movwf	ee_data	
	return
;------------------------------------------------------------

delay_256
        movlw   0xff                    ;256 mS delay
        movwf   delay_count
	movwf	count_1
delay_loop
	nop
	decfsz	count_1,F
	goto	delay_loop
        decfsz  delay_count,F
        goto    delay_loop
        return
;------------------------------------------------------------
        end
;------------------------------------------------------------
Copyright (c) 2002, Mike Berg 
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions 
are met:

Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 
Redistributions in binary form must reproduce the above copyright 
notice, this list of conditions and the following disclaimer in the 
documentation and/or other materials provided with the distribution. 
Neither the name of the <ORGANIZATION> nor the names of its 
contributors may be used to endorse or promote products derived from 
this software without specific prior written permission. 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.

